@@ -1,8 +1,4 @@
 """ interactive debugging with PDB, the Python Debugger. """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import argparse
 import pdb
 import sys
@@ -48,42 +44,18 @@ def pytest_addoption(parser):
     )
 
 
-def _import_pdbcls(modname, classname):
-    try:
-        __import__(modname)
-        mod = sys.modules[modname]
-
-        # Handle --pdbcls=pdb:pdb.Pdb (useful e.g. with pdbpp).
-        parts = classname.split(".")
-        pdb_cls = getattr(mod, parts[0])
-        for part in parts[1:]:
-            pdb_cls = getattr(pdb_cls, part)
-
-        return pdb_cls
-    except Exception as exc:
-        value = ":".join((modname, classname))
-        raise UsageError("--pdbcls: could not import {!r}: {}".format(value, exc))
-
-
 def pytest_configure(config):
-    pdb_cls = config.getvalue("usepdb_cls")
-    if pdb_cls:
-        pdb_cls = _import_pdbcls(*pdb_cls)
-    else:
-        pdb_cls = pdb.Pdb
-
     if config.getvalue("trace"):
         config.pluginmanager.register(PdbTrace(), "pdbtrace")
     if config.getvalue("usepdb"):
         config.pluginmanager.register(PdbInvoke(), "pdbinvoke")
 
     pytestPDB._saved.append(
-        (pdb.set_trace, pytestPDB._pluginmanager, pytestPDB._config, pytestPDB._pdb_cls)
+        (pdb.set_trace, pytestPDB._pluginmanager, pytestPDB._config)
     )
     pdb.set_trace = pytestPDB.set_trace
     pytestPDB._pluginmanager = config.pluginmanager
     pytestPDB._config = config
-    pytestPDB._pdb_cls = pdb_cls
 
     # NOTE: not using pytest_unconfigure, since it might get called although
     #       pytest_configure was not (if another plugin raises UsageError).
@@ -92,20 +64,19 @@ def pytest_configure(config):
             pdb.set_trace,
             pytestPDB._pluginmanager,
             pytestPDB._config,
-            pytestPDB._pdb_cls,
         ) = pytestPDB._saved.pop()
 
     config._cleanup.append(fin)
 
 
-class pytestPDB(object):
+class pytestPDB:
     """ Pseudo PDB that defers to the real pdb. """
 
     _pluginmanager = None
     _config = None
-    _pdb_cls = pdb.Pdb
     _saved = []
     _recursive_debug = 0
+    _wrapped_pdb_cls = None
 
     @classmethod
     def _is_capturing(cls, capman):
@@ -114,16 +85,138 @@ class pytestPDB(object):
         return False
 
     @classmethod
-    def _init_pdb(cls, *args, **kwargs):
+    def _import_pdb_cls(cls, capman):
+        if not cls._config:
+            # Happens when using pytest.set_trace outside of a test.
+            return pdb.Pdb
+
+        usepdb_cls = cls._config.getvalue("usepdb_cls")
+
+        if cls._wrapped_pdb_cls and cls._wrapped_pdb_cls[0] == usepdb_cls:
+            return cls._wrapped_pdb_cls[1]
+
+        if usepdb_cls:
+            modname, classname = usepdb_cls
+
+            try:
+                __import__(modname)
+                mod = sys.modules[modname]
+
+                # Handle --pdbcls=pdb:pdb.Pdb (useful e.g. with pdbpp).
+                parts = classname.split(".")
+                pdb_cls = getattr(mod, parts[0])
+                for part in parts[1:]:
+                    pdb_cls = getattr(pdb_cls, part)
+            except Exception as exc:
+                value = ":".join((modname, classname))
+                raise UsageError(
+                    "--pdbcls: could not import {!r}: {}".format(value, exc)
+                )
+        else:
+            pdb_cls = pdb.Pdb
+
+        wrapped_cls = cls._get_pdb_wrapper_class(pdb_cls, capman)
+        cls._wrapped_pdb_cls = (usepdb_cls, wrapped_cls)
+        return wrapped_cls
+
+    @classmethod
+    def _get_pdb_wrapper_class(cls, pdb_cls, capman):
+        import _pytest.config
+
+        class PytestPdbWrapper(pdb_cls):
+            _pytest_capman = capman
+            _continued = False
+
+            def do_debug(self, arg):
+                cls._recursive_debug += 1
+                ret = super().do_debug(arg)
+                cls._recursive_debug -= 1
+                return ret
+
+            def do_continue(self, arg):
+                ret = super().do_continue(arg)
+                if cls._recursive_debug == 0:
+                    tw = _pytest.config.create_terminal_writer(cls._config)
+                    tw.line()
+
+                    capman = self._pytest_capman
+                    capturing = pytestPDB._is_capturing(capman)
+                    if capturing:
+                        if capturing == "global":
+                            tw.sep(">", "PDB continue (IO-capturing resumed)")
+                        else:
+                            tw.sep(
+                                ">",
+                                "PDB continue (IO-capturing resumed for %s)"
+                                % capturing,
+                            )
+                        capman.resume()
+                    else:
+                        tw.sep(">", "PDB continue")
+                cls._pluginmanager.hook.pytest_leave_pdb(config=cls._config, pdb=self)
+                self._continued = True
+                return ret
+
+            do_c = do_cont = do_continue
+
+            def do_quit(self, arg):
+                """Raise Exit outcome when quit command is used in pdb.
+
+                This is a bit of a hack - it would be better if BdbQuit
+                could be handled, but this would require to wrap the
+                whole pytest run, and adjust the report etc.
+                """
+                ret = super().do_quit(arg)
+
+                if cls._recursive_debug == 0:
+                    outcomes.exit("Quitting debugger")
+
+                return ret
+
+            do_q = do_quit
+            do_exit = do_quit
+
+            def setup(self, f, tb):
+                """Suspend on setup().
+
+                Needed after do_continue resumed, and entering another
+                breakpoint again.
+                """
+                ret = super().setup(f, tb)
+                if not ret and self._continued:
+                    # pdb.setup() returns True if the command wants to exit
+                    # from the interaction: do not suspend capturing then.
+                    if self._pytest_capman:
+                        self._pytest_capman.suspend_global_capture(in_=True)
+                return ret
+
+            def get_stack(self, f, t):
+                stack, i = super().get_stack(f, t)
+                if f is None:
+                    # Find last non-hidden frame.
+                    i = max(0, len(stack) - 1)
+                    while i and stack[i][0].f_locals.get("__tracebackhide__", False):
+                        i -= 1
+                return stack, i
+
+        return PytestPdbWrapper
+
+    @classmethod
+    def _init_pdb(cls, method, *args, **kwargs):
         """ Initialize PDB debugging, dropping any IO capturing. """
         import _pytest.config
 
         if cls._pluginmanager is not None:
             capman = cls._pluginmanager.getplugin("capturemanager")
-            if capman:
-                capman.suspend(in_=True)
+        else:
+            capman = None
+        if capman:
+            capman.suspend(in_=True)
+
+        if cls._config:
             tw = _pytest.config.create_terminal_writer(cls._config)
             tw.line()
+
             if cls._recursive_debug == 0:
                 # Handle header similar to pdb.set_trace in py37+.
                 header = kwargs.pop("header", None)
@@ -131,102 +224,32 @@ class pytestPDB(object):
                     tw.sep(">", header)
                 else:
                     capturing = cls._is_capturing(capman)
-                    if capturing:
-                        if capturing == "global":
-                            tw.sep(">", "PDB set_trace (IO-capturing turned off)")
-                        else:
-                            tw.sep(
-                                ">",
-                                "PDB set_trace (IO-capturing turned off for %s)"
-                                % capturing,
-                            )
+                    if capturing == "global":
+                        tw.sep(">", "PDB {} (IO-capturing turned off)".format(method))
+                    elif capturing:
+                        tw.sep(
+                            ">",
+                            "PDB %s (IO-capturing turned off for %s)"
+                            % (method, capturing),
+                        )
                     else:
-                        tw.sep(">", "PDB set_trace")
+                        tw.sep(">", "PDB {}".format(method))
 
-            class PytestPdbWrapper(cls._pdb_cls, object):
-                _pytest_capman = capman
-                _continued = False
+        _pdb = cls._import_pdb_cls(capman)(**kwargs)
 
-                def do_debug(self, arg):
-                    cls._recursive_debug += 1
-                    ret = super(PytestPdbWrapper, self).do_debug(arg)
-                    cls._recursive_debug -= 1
-                    return ret
-
-                def do_continue(self, arg):
-                    ret = super(PytestPdbWrapper, self).do_continue(arg)
-                    if cls._recursive_debug == 0:
-                        tw = _pytest.config.create_terminal_writer(cls._config)
-                        tw.line()
-
-                        capman = self._pytest_capman
-                        capturing = pytestPDB._is_capturing(capman)
-                        if capturing:
-                            if capturing == "global":
-                                tw.sep(">", "PDB continue (IO-capturing resumed)")
-                            else:
-                                tw.sep(
-                                    ">",
-                                    "PDB continue (IO-capturing resumed for %s)"
-                                    % capturing,
-                                )
-                            capman.resume()
-                        else:
-                            tw.sep(">", "PDB continue")
-                    cls._pluginmanager.hook.pytest_leave_pdb(
-                        config=cls._config, pdb=self
-                    )
-                    self._continued = True
-                    return ret
-
-                do_c = do_cont = do_continue
-
-                def do_quit(self, arg):
-                    """Raise Exit outcome when quit command is used in pdb.
-
-                    This is a bit of a hack - it would be better if BdbQuit
-                    could be handled, but this would require to wrap the
-                    whole pytest run, and adjust the report etc.
-                    """
-                    ret = super(PytestPdbWrapper, self).do_quit(arg)
-
-                    if cls._recursive_debug == 0:
-                        outcomes.exit("Quitting debugger")
-
-                    return ret
-
-                do_q = do_quit
-                do_exit = do_quit
-
-                def setup(self, f, tb):
-                    """Suspend on setup().
-
-                    Needed after do_continue resumed, and entering another
-                    breakpoint again.
-                    """
-                    ret = super(PytestPdbWrapper, self).setup(f, tb)
-                    if not ret and self._continued:
-                        # pdb.setup() returns True if the command wants to exit
-                        # from the interaction: do not suspend capturing then.
-                        if self._pytest_capman:
-                            self._pytest_capman.suspend_global_capture(in_=True)
-                    return ret
-
-            _pdb = PytestPdbWrapper(**kwargs)
+        if cls._pluginmanager:
             cls._pluginmanager.hook.pytest_enter_pdb(config=cls._config, pdb=_pdb)
-        else:
-            _pdb = cls._pdb_cls(**kwargs)
         return _pdb
 
     @classmethod
     def set_trace(cls, *args, **kwargs):
         """Invoke debugging via ``Pdb.set_trace``, dropping any IO capturing."""
         frame = sys._getframe().f_back
-        _pdb = cls._init_pdb(*args, **kwargs)
+        _pdb = cls._init_pdb("set_trace", *args, **kwargs)
         _pdb.set_trace(frame)
 
 
-class PdbInvoke(object):
+class PdbInvoke:
     def pytest_exception_interact(self, node, call, report):
         capman = node.config.pluginmanager.getplugin("capturemanager")
         if capman:
@@ -241,7 +264,7 @@ class PdbInvoke(object):
         post_mortem(tb)
 
 
-class PdbTrace(object):
+class PdbTrace:
     @hookimpl(hookwrapper=True)
     def pytest_pyfunc_call(self, pyfuncitem):
         _test_pytest_function(pyfuncitem)
@@ -249,7 +272,7 @@ class PdbTrace(object):
 
 
 def _test_pytest_function(pyfuncitem):
-    _pdb = pytestPDB._init_pdb()
+    _pdb = pytestPDB._init_pdb("runcall")
     testfunction = pyfuncitem.obj
     pyfuncitem.obj = _pdb.runcall
     if "func" in pyfuncitem._fixtureinfo.argnames:  # pragma: no branch
@@ -298,22 +321,8 @@ def _postmortem_traceback(excinfo):
         return excinfo._excinfo[2]
 
 
-def _find_last_non_hidden_frame(stack):
-    i = max(0, len(stack) - 1)
-    while i and stack[i][0].f_locals.get("__tracebackhide__", False):
-        i -= 1
-    return i
-
-
 def post_mortem(t):
-    class Pdb(pytestPDB._pdb_cls, object):
-        def get_stack(self, f, t):
-            stack, i = super(Pdb, self).get_stack(f, t)
-            if f is None:
-                i = _find_last_non_hidden_frame(stack)
-            return stack, i
-
-    p = Pdb()
+    p = pytestPDB._init_pdb("post_mortem")
     p.reset()
     p.interaction(None, t)
     if p.quitting:
